package consume

import (
	"encoding/json"
	"fmt"
	"query-service/internal/adapters/brokeradapter"
	"query-service/pkg/errorhandler"
	"query-service/pkg/logger"
)

// HandleMessage gets called when a message is received
func (s *Service) HandleMessage(msg *brokeradapter.Message) {
	// Grab sessionID from the headers
	sessionID, ok := msg.Headers["sessionID"].(string)
	if !ok {
		// TODO: Handle error where there is no session ID supplied
	}

	// Convert the json byte msg to a query string
	query, err := s.queryConverter.ConvertQuery(&msg.Body)
	if err != nil {
		errorhandler.LogError(err, "failed to parse incoming msg to query language") // TODO: don't panic on error, send error message to client instead
		return
	}
	fmt.Println("Query: " + *query)

	// Make request to database
	// TODO : Generate database seperately
	// execute and retrieve result
	// convert result to general (node-link (?)) format
	result, err := s.requestSender.SendAQLQuery(*query)
	if err != nil {
		logger.Log(err.Error())
		return // TODO: Send message in queue notifying of error
	}

	// Add type indicator to result from database
	querymap := make(map[string]interface{})
	querymap["type"] = "query_result"
	querymap["pod"] = podName
	querymap["values"] = *result
	querybyte, err := json.Marshal(querymap)
	//fmt.Println(querymap)

	s.producer.PublishMessage(&querybyte, &sessionID)
}
